'use strict'

import React, { Component } from 'react'
import PropTypes from 'prop-types'

class VroomAuthLogin extends Component {
    constructor(props) {
        super(props)
        this.signIn = this.signIn.bind(this)
        this.state = {
            disabled: true
        }
    }

    componentDidMount() {
        const { clientId, cookiePolicy, loginHint, hostedDomain, autoLoad, fetchBasicProfile, redirectUri, discoveryDocs, onFailure, uiMode} = this.props;
        ((doc, tag, id, callback) => {
            const element = doc.getElementsByTagName(tag)[0]
            const fjs = element;
            let js = element;
            js = doc.createElement(tag);
            js.id = id;
            js.src = '//apis.google.com/js/api:client.js';
            fjs.parentNode.insertBefore(js, fjs);
            js.onload = callback;
        })(document, 'script', 'google-login', () => {
            const params = {
                client_id: clientId,
                cookiepolicy: cookiePolicy,
                login_hint: loginHint,
                hosted_domain: hostedDomain,
                fetch_basic_profile: fetchBasicProfile,
                discoveryDocs,
                ui_mode: uiMode,
                redirect_uri: redirectUri
            };
            window.gapi.load('auth2', () => {
                this.setState({
                    disabled: false,
                });
                if (!window.gapi.auth2.getAuthInstance()) {
                    window.gapi.auth2.init(params)
                        .then((res) => {
                            if (isSignedIn && res.isSignedIn.get()) {
                                this._handleSigninSuccess(res.currentUser.get())
                            }
                        },
                        err => onFailure(err)
                    )
                }
                if (autoLoad) {
                    this.signIn()
                }
            })
        });
    }

    signIn(e) {
        if (e) {
            e.preventDefault()
        }
        if (!this.state.disabled) {
            const auth2 = window.gapi.auth2.getAuthInstance();
            const { redirectUri, onSuccess, onRequest, fetchBasicProfile, onFailure, prompt, scope, offlineMode } = this.props;
            const options = {
                offline_mode: offlineMode,
                redirect_uri: redirectUri,
                fetch_basic_profile: fetchBasicProfile,
                prompt,
                scope
            };
            onRequest();
            if (offlineMode) {
                auth2.grantOfflineAccess(options)
                    .then(
                        res => onSuccess(res),
                        err => onFailure(err)
                    );
            } else {
                auth2.signIn(options)
                    .then(
                        res => this._handleSigninSuccess(res),
                        err => onFailure(err)
                    );
            }
        }
    }
    _handleSigninSuccess(res) {
        const basicProfile = res.getBasicProfile()
        const authResponse = res.getAuthResponse()
        res.googleId = basicProfile.getId()
        res.tokenObj = authResponse
        res.tokenId = authResponse.id_token
        res.accessToken = authResponse.access_token
        res.profileObj = {
            googleId: basicProfile.getId(),
            imageUrl: basicProfile.getImageUrl(),
            email: basicProfile.getEmail(),
            name: basicProfile.getName(),
            givenName: basicProfile.getGivenName(),
            familyName: basicProfile.getFamilyName()
        }
        this.props.onSuccess(res)
    }

    render () {
        const { tag, style, className, disabledStyle, buttonText, children } = this.props
        const disabled = this.state.disabled || this.props.disabled
        const initialStyle = {
            display: 'inline-block',
            background: '#ffffff',
            color: '#4097D3',
            width: 190,
            paddingTop: 10,
            paddingBottom: 10,
            borderRadius: 2,
            border: '1px solid gainsboro',
            fontSize: 16,
            fontWeight: 'bold',
            fontFamily: 'BlinkMacSystemFont, -apple-system, "Segoe UI", "Roboto", "Oxygen", "Ubuntu", "Cantarell", "Fira Sans", "Droid Sans", "Helvetica Neue", "Helvetica", "Arial", sans-serif'
        };
        const styleProp = (() => {
            if (style) {
                return style;
            } else if (className && !style) {
                return {};
            }
            return initialStyle;
        })();
        const defaultStyle = (() => {
            if (disabled) {
                return Object.assign({}, styleProp, disabledStyle);
            }
            return styleProp;
        })();
        const vroomAuthLoginButton = React.createElement(
            tag, {
                onClick: this.signIn,
                style: defaultStyle,
                disabled,
                className,
            }, children ? children : buttonText
        );
        return vroomAuthLoginButton
    }
}

VroomAuthLogin.propTypes = {
    onSuccess: PropTypes.func.isRequired,
    onFailure: PropTypes.func.isRequired,
    clientId: PropTypes.string.isRequired,
    onRequest: PropTypes.func,
    buttonText: PropTypes.string,
    scope: PropTypes.string,
    className: PropTypes.string,
    redirectUri: PropTypes.string,
    cookiePolicy: PropTypes.string,
    loginHint: PropTypes.string,
    hostedDomain: PropTypes.string,
    children: PropTypes.node,
    style: PropTypes.object,
    disabledStyle: PropTypes.object,
    fetchBasicProfile: PropTypes.bool,
    prompt: PropTypes.string,
    tag: PropTypes.string,
    autoLoad: PropTypes.bool,
    disabled: PropTypes.bool,
    discoveryDocs: PropTypes.array,
    offlineMode: PropTypes.bool,
    uiMode: PropTypes.string,
    isSignedIn: PropTypes.bool,
}

VroomAuthLogin.defaultProps = {
    isSignedIn: false,
    tag: 'button',
    buttonText: 'Sign In with Google',
    scope: 'profile email',
    offlineMode: false,
    prompt: '',
    hostedDomain: 'vroom.com',
    cookiePolicy: 'single_host_origin',
    fetchBasicProfile: true,
    uiMode: 'popup',
    disabledStyle: {
        opacity: 0.5
    },
    onRequest: () => {}
}

export default VroomAuthLogin;